﻿using EvaPOS_API_FRAME.Comandos;
using EvaPOS_API_FRAME.DTO;
using EvaPOS_API_FRAME.RespuestasXML;
using EvaPOS_API_FRAME.RespuestasXML.Suspend;
using EvaPosSrvDTO;
using EvaPosSrvResp;
using gatewayGK.POSGk;
using gatewayGK.RespuestasXML.Suspend;
using RestSharp;
using SCOGateway.POSGk;
using Serilog;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace gatewayGK.ComandosGkPruebas
{
    ///<summary>
    /// Comando para suspender transacción
    /// </summary>
    public class SuspendTransactionCmdGK : IComando
    {
        public string Referencia { get; set; } = "scsns:Suspend";

        /// <summary>
        /// DTO con solicitud.
        /// </summary>
        public SuspendTransactionRequest Request { get; private set; }
        /// <summary>
        /// Procesa y responde SuspendTransactionRequest.
        /// </summary>
        public const string UrlSuspend = ConstantesGK.UrlBaseMaqueta + "/com.gk_software.pos.api.service.transaction.PosTransactionService/suspendTransaction";
        /// <summary>
        /// Url servicio subtotal Gk.
        /// </summary>

        public Respuestas Ejecutar()
        {
            Log.Debug("Cmd SuspendTransactionRequesttCmd ejecutado. SuspendTransactionRequest : {SuspendTransactionRequest}", Request.RequestID);
            int requestId = Request.RequestID;
            Respuestas respuesta = null;

            var suspend = new RestClient(UrlSuspend);
            var request = new RestRequest(UrlSuspend, Method.Post);
            var suspendDatos = new SuspendTransactionReq { SuspendNumber = "1234", BarCodePrefix = "1234"};
            request.AddJsonBody(suspendDatos);

            // Ejecuta invocación sincrónica, deserializando respuesta según el tipo indicado en la invocación del Post<T>.
            var suspenderResp = suspend.Post<SuspendTransactionResp>(request) ?? throw new ApplicationException("Servicio POST 'Suspender' retorna nulos.");
            Log.Debug($"GkPruebas: Respuesta Cliente - POST - response {suspenderResp}");
            if ( suspenderResp != null)
            {
                var suspendTransaction = new SuspendTransactionResponse(1, TipoMensaje.Resp, requestId);
                respuesta = new Respuestas { suspendTransaction };

            }
            else
            {
                var suspendTransaction = new SuspendTransactionResponseError(1, TipoMensaje.Resp, requestId, "No se puede realizar la suspension", "APPLICATION_NOT_IN_PROPER_STATE: TRANSACTION_NOT_ACTIVE");
                respuesta = new Respuestas { suspendTransaction };
            }

            return respuesta;
        }

        public IComando CreaCopia()
        {
            return (SuspendTransactionRequestCmd)this.MemberwiseClone();
        }
        public void CargaDTO(DTOBase suspendTransactionRequest)
        {
            Request = (SuspendTransactionRequest)suspendTransactionRequest;
        }
    }
}
